# This is released under the LGPL for everyone's use. Not warrenty etc.
# Copyright by Mark Zealey (aka JALH on opn) (mark@itsolve.co.uk). Please
# credit me with it if ya make changes etc in accordance with the LGPL

package JALH::Client;
use strict;

=head1 NAME

JALH::Client - A client module

=head1 SYNOPSIS

    use JALH::Client;

    $client = new JALH::Client(	Server => 'foobar.com',
				Port => 6666,
				Proto => 'tcp' );
    my $ch = $client->connect;
    print $ch "Hello server!\n";
    $ch = $client->change_server(	Reconnect => 1,
					Server => 'barfoo.com',
					Port => '80' );
    my $response = <$ch>;
    $client->disconnect;

=head1 DESCRIPTION

C<JALH::Client> connects to any server, (maybe one made by C<JALH::Server> ;)
and then allows you to send and recieve commands to it via the filehandle that
it returns.

=cut

require Exporter;
use Socket;
use Carp;
use IO::Handle;

# Exporter stuff
my @ISA = qw(Exporter);
my @EXPORT = qw(new disconnect connect change_server disconnected);
my @EXPORT_OK = qw();

=head1 SERVER OBJECT

=head2 CONSTRUCTOR

=over 4

=item new([I<ARGS>])

Creates a new client. The object can take the following parameters.

    Name	Description		Default
    Port	Port number		[12345]
    Proto	The protocol		['tcp']
    Server	The server's URL/IP	['localhost']

=back

=cut

# Creates the new instance
sub new {
	my $t = shift;
	my $class = ref($t) || $t;
	my $self = {};

	bless $self, $class;

	$self->change_server(@_);

	return $self;
}

=head2 METHODS

=over 4

=item connect()

Connects to the server and returns a filehandle (or undef if the connection
fails)

=cut

sub connect {
	my $self = shift;

	if(!socket S, PF_INET, SOCK_STREAM, getprotobyname $self->{Proto}) {
		carp "Socket: $!";
		$self->{Connected} = 0;
		return undef;		# Fail
	}

	$self->{Sock} = *S;

	if(!connect $self->{Sock}, sockaddr_in($self->{Port},
						inet_aton $self->{Server})) {
		carp "Connect: $!";
		$self->{Connected} = 0;
		return undef;		# Fail :/
	}

	$self->{Connected} = 1;
	$self->{Sock}->autoflush(1);		# To avoid nasty write problems
	return $self->{Sock};
}

=item disconnect()

Disconnects from the server

=cut

sub disconnect {
	my $self = shift;

	close $self->{Sock};
	$self->{Connected} = 0;
}

=item disconnected()

Returns 1 if you have been disconnected from the server.

=cut

sub disconnected {
	my $self = shift;

	return eof $self->{Sock}
}

=item change_server([I<ARGS>])

Disconnects from the server and changes the server details to a new one. If
reconnect is non-zero, it reconnects and returns the handle.

Arguments are:
    Name	Description			Default
    Port	Port number			12345
    Proto	The protocol			'tcp'
    Server	The server's URL/IP		'localhost'
    Reconnect	Should I reconnect (0 for no,	0
		non-zero for yes)

=cut

sub change_server {
	my $self = shift;
	my %arg = @_;

	$self->disconnect if $self->{Connected};

	$self->{Port} = $arg{Port} || 12345;
	$self->{Proto} = $arg{Proto} || 'tcp';
	$self->{Server} = $arg{Server} || 'localhost';

	# Convert the port to a number if it isnt allready
	$self->{Port} = getservbyname $self->{Port}, $self->{Proto}
						if $self->{Port} =~ /\D/;
	carp "No port" unless $self->{Port};

	return $self->connect if exists $arg{Reconnect} && $arg{Reconnect};
	
	return undef;			# Else
}

=back

=head1 AUTHOR

Mark Zealey <mark@itsolve.co.uk>

=head1 COPYRIGHT

Copyright 2000 Mark Zealey. All rights reserved. blah blah blah

This program is free software released under the LGPL.

=cut

1
