# This is released under the LGPL for everyone's use. Not warrenty etc.
# Copyright by Mark Zealey (aka JALH on opn) (mark@itsolve.co.uk). Please
# credit me with it if ya make changes etc in accordance with the LGPL

package JALH::Server;
use strict;

=head1 NAME

JALH::Server - A multi-threaded server module

=head1 SYNOPSIS

    use JALH::Server;

    my $server = new JALH::Server(	Port => 6666,
					Spawn => \&client,
					);
    $server->start;

    sub client {
	my ($self, $handle, $paddr) = @_;
	chomp(my $t = <$handle>);
	print $handle "$t\n";
	$self->killall if lc $t eq 'killall';
	$self->quit if lc $t eq 'quit';
    }

=head1 DESCRIPTION

C<JALH::Server> provides a means to make a multi-threaded, portable server in
only a few lines of perl code.

=cut

require Exporter;
use Socket;
use Carp;

# Exporter stuff
my @ISA = qw(Exporter);
my @EXPORT = qw(new start);
my @EXPORT_OK = qw();

my $waitedpid;

BEGIN {
	$SIG{CHLD} = \&REAPER
}

=head1 SERVER OBJECT

=head2 CONSTRUCTOR

=over 4

=item new([I<ARGS>])

Creates a new server. The object can take the following parameters.

    Name	Description			Default
    Port	Port number			12345
    Proto	The protocol			'tcp'
    Interface	The interface			[kernel decides]
    Spawn	A referance to the sub to spawn	[Required]
		on each new connection
    QueueSize	The queue size			SOMAXCONN [Max pos]

=back

=cut

# Creates the new instance
sub new {
	my $t = shift;
	my $class = ref($t) || $t;
	my %arg = @_;
	confess "No handler specified with the Spawn argument"
			unless exists $arg{Spawn} && $arg{Spawn}
				&& ref($arg{Spawn}) eq 'CODE';
	
	my $self = {
			Port => $arg{Port} || 12345,
			Proto => $arg{Proto} || 'tcp',
			Spawn => $arg{Spawn},
			IF => $arg{Interface} || INADDR_ANY,
			QS => $arg{QueueSize} || SOMAXCONN,
			Quit => 0,		# Dont quit
			Clients => [],
	};

	$self->{Port} = $1 if $self->{Port} =~ /(\d+)/;

	socket S, PF_INET, SOCK_STREAM, getprotobyname $self->{Proto}
						or croak "socket: $!";
	$self->{Sock} = *S;

	setsockopt $self->{Sock}, SOL_SOCKET, SO_REUSEADDR, pack("l", 1)
						or croak "setsockopt: $!";

	bind $self->{Sock}, sockaddr_in($self->{Port}, $self->{IF})
						or croak "bind: $!";

	listen $self->{Sock}, $self->{QS}	or croak "listen: $!";

	bless $self, $class;
	
	return $self;
}

sub REAPER {
	$waitedpid = wait;
	$SIG{CHLD} = \&REAPER;
}

sub spawn {
	if (!defined(my $pid = fork)) {
		carp "cannot fork: $!";
		return;
	} elsif ($pid) {
		return; # I'm the parent
	}
	# else I'm the child - spawn :)
	exit &{$_[0]->{Spawn}}(@_);
}

=head2 METHODS

=over 4

=item start()

Starts the server going after you created the new server object. Returns on a
quit()

(Each spawn is called with the args $self, A referance to the client's handle
and a paddr, as returned from 'accept')

=cut

sub start {
	my $self = shift;
	my $paddr;
	$waitedpid = 0;
	while($self->{Quit} == 0) {
		($paddr = accept Client, $self->{Sock}) or next;
		last if $self->{Quit} != 0;
		push @{$self->{Clients}}, *Client;
		spawn($self, *Client, $paddr);
		$waitedpid = 0;
		$self->disconnect(*Client);
	}

	# Dont accept new connections (but dont return either)
	while($self->{Quit} == 1) {}	
}

=item killall()

Kills all clients from the server

=cut

sub killall {
	my $self = shift;
	close $_ for @{$self->{Clients}};	# Quick method :)
	$self->{Clients} = [];
}

=item connected()

Returns the number of clients currently connected to the server.

=cut

sub connected {
	return scalar @{$_[0]->{Clients}};
}

=item disconnect(*Client)

Disconnects a client from the server

=cut

sub disconnect {
	my $self = shift;
	my $ch = shift;

	close $ch;
	@{$self->{Clients}} = grep $_ ne $ch, @{$self->{Clients}};
}

=item quit()

Closes the server down and terminates all client connections

=cut

sub quit {
	my $self = shift;
	$self->{Quit} = 1;		# Disallow all new connections
	$self->killall;			# Kill all clients
	close $self->{Sock};		# Close the server
	$self->{Quit} = 2;		# Exit the infinate loop
}

=back

=head1 AUTHOR

Mark Zealey <mark@itsolve.co.uk>

=head1 COPYRIGHT

Copyright 2000 Mark Zealey. All rights reserved. blah blah blah

This program is free software released under the LGPL.

=cut

"As bod suggested, almost anything is non-zero, including this ;)"
